const {
  services,
  Email,
  Fragment,
  Invitation,
} = require('pubsweet-component-helper-service')

module.exports = models => async (req, res) => {
  const { collectionId, invitationId, fragmentId } = req.params
  const { invitationToken } = req.body

  if (!services.checkForUndefinedParams(invitationToken))
    return res.status(400).json({ error: 'Token is required' })

  const UserModel = models.User
  try {
    const user = await UserModel.findOneByField(
      'invitationToken',
      invitationToken,
    )
    const collection = await models.Collection.find(collectionId)
    if (!collection.fragments.includes(fragmentId))
      return res.status(400).json({
        error: `Fragment ${fragmentId} does not match collection ${collectionId}`,
      })
    const fragment = await models.Fragment.find(fragmentId)
    fragment.invitations = fragment.invitations || []
    const invitation = await fragment.invitations.find(
      invitation => invitation.id === invitationId,
    )

    const invitationHelper = new Invitation({
      userId: user.id,
      role: 'reviewer',
    })

    const invitationValidation = invitationHelper.validateInvitation({
      invitation,
    })
    if (invitationValidation.error)
      return res.status(invitationValidation.status).json({
        error: invitationValidation.error,
      })

    invitation.respondedOn = Date.now()
    invitation.hasAnswer = true
    invitation.isAccepted = false
    await fragment.save()

    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const baseUrl = services.getBaseUrl(req)
    const {
      authorsList: authors,
      submittingAuthor,
    } = await fragmentHelper.getAuthorData({ UserModel })
    const emailHelper = new Email({
      UserModel,
      collection,
      parsedFragment,
      baseUrl,
      authors,
    })
    emailHelper.setupReviewerDecisionEmail({
      authorName: `${submittingAuthor.firstName} ${submittingAuthor.lastName}`,
      agree: false,
      user,
    })
    return res.status(200).json({})
  } catch (e) {
    const notFoundError = await services.handleNotFoundError(e, 'item')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
